// kernel.c
#include "string.h"
#include "kernel.h"
#include "memory.c"
#include "process.c"
#include "filesystem.c"
#include "mutex.c"
#include "sync.c"

typedef unsigned int uint32_t;

// 向 UART 发送单个字符
void uart_putc(char c) {
    volatile char *uart = (volatile char *)UART_BASE;
    *uart = c;
}

// 输出字符串
void uart_puts(const char *s) {
    while (*s) {
        uart_putc(*s++);
    }
}

// 从 UART 接收单个字符
char uart_getc() {
    volatile char *uart = (volatile char *)UART_BASE;
    volatile char *lsr = (volatile char *)UART_LSR;
    while (!(*lsr & UART_LSR_DR));
    return *uart;
}

// 清屏函数
void clear_screen() {
    for (int i = 0; i < 50; i++) {
        uart_putc('\n');
    }
}

// 输出命令提示符
void print_prompt() {
    uart_puts("\nOS> ");
}



// 命令处理函数
void handle_command(char *input) {
    if (strcmp_(input, "help") == 0) {
        uart_puts("\nAvailable commands:\n");
        uart_puts("  help        - Show available commands\n");
        uart_puts("  malloc <n>  - Allocate n bytes of memory\n");
        uart_puts("  free        - Free all allocated memory\n");
        uart_puts("  run <n>     - Run tasks n times\n");
        uart_puts("  clear       - Clear the screen\n");
        uart_puts("  about       - Show OS information\n");
        uart_puts("  exit        - Exit the program\n");
        uart_puts("  fs_init     - Initialize the file system\n");
        uart_puts("  create <name> - Create a file\n");
        uart_puts("  delete <name> - Delete a file\n");
        uart_puts("  write <name> <data> - Write data to a file\n");
        uart_puts("  read <name> - Read data from a file\n");
        uart_puts("  list        - List all files\n");
        uart_puts("  mutex_demo  - Demonstrate mutex lock/unlock\n");
        uart_puts("  sync_demo   - Demonstrate semaphore wait/signal\n");
    } else if (strcmp_(input, "clear") == 0) {
        clear_screen();
    } else if (strcmp_(input, "about") == 0) {
        uart_puts("\nvamOS\nVersion 1.0\nDeveloped for educational purposes.\n");
    } else if (strncmp_(input, "malloc ", 7) == 0) {
        int size = 0;
        const char *ptr = input + 7;
        while (*ptr >= '0' && *ptr <= '9') {
            size = size * 10 + (*ptr - '0');
            ptr++;
        }
        void *allocated = simple_malloc(size);
        if (allocated) {
            uart_puts("Memory allocated.\n");
        } else {
            uart_puts("Memory allocation failed.\n");
        }
    } else if (strcmp_(input, "free") == 0) {
        simple_free();
        uart_puts("Memory freed.\n");
    } else if (strncmp_(input, "run ", 4) == 0) {
        int run_count = 0;
        const char *ptr = input + 4;
        while (*ptr >= '0' && *ptr <= '9') {
            run_count = run_count * 10 + (*ptr - '0');
            ptr++;
        }
        if (run_count > 0) {
            uart_puts("Starting tasks...\n");
            schedule(run_count);
        } else {
            uart_puts("Invalid run count. Please specify a positive number.\n");
        }
    } else if (strcmp_(input, "exit") == 0) {
        uart_puts("Exiting vamOS...\n");
        volatile uint32_t *exit_signal = (volatile uint32_t *)0x100000;
        *exit_signal = 0x10;
        while (1);
    } else if (strcmp_(input, "fs_init") == 0) {
        fs_init();
    } else if (strncmp_(input, "create ", 7) == 0) {
        fs_create(input + 7);
    } else if (strncmp_(input, "delete ", 7) == 0) {
        fs_delete(input + 7);
    } else if (strncmp_(input, "write ", 6) == 0) {
        char *filename = input + 6;
        char *data = strchr_(filename, ' ');
        if (data) {
            *data = '\0';
            data++;
            fs_write(filename, data, strlen_(data));
        } else {
            uart_puts("Invalid write command format.\n");
        }
    } else if (strncmp_(input, "read ", 5) == 0) {
        char buffer[256];
        fs_read(input + 5, buffer, sizeof(buffer));
        uart_puts("File content: ");
        uart_puts(buffer);
        uart_putc('\n');
    } else if (strcmp_(input, "list") == 0) {
        fs_list();
    } else if (strcmp_(input, "mutex_demo") == 0) {
        mutex_demo();
    } else if (strcmp_(input, "sync_demo") == 0) {
        sync_demo();
    } else {
        uart_puts("\nUnknown command. Type 'help' for a list of commands.\n");
    }
}

// 主函数
void main() {
    uart_puts("Welcome to vamOS!\n");
    uart_puts("Type 'help' for a list of commands.\n");

    char input_buffer[128];
    int index = 0;

    print_prompt();

    while (1) {
        char c = uart_getc();
        if (c == '\r') {
            uart_putc('\n');
            input_buffer[index] = '\0';
            handle_command(input_buffer);
            index = 0;
            print_prompt();
        } else if (c == '\b') {
            if (index > 0) {
                index--;
                uart_putc('\b');
                uart_putc(' ');
                uart_putc('\b');
            }
        } else if (index < sizeof(input_buffer) - 1) {
            input_buffer[index++] = c;
            uart_putc(c);
        }
    }
}
